@@ -5,18 +5,20 @@ use Mouse;
 use Lemonldap::NG::Portal::Lib::SAML;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
+  PE_SESSIONEXPIRED
   PE_SAML_ART_ERROR
   PE_SAML_DESTINATION_ERROR
   PE_SAML_SESSION_ERROR
   PE_SAML_SIGNATURE_ERROR
   PE_SAML_SLO_ERROR
   PE_SAML_SSO_ERROR
+  PE_ISSUERMISSINGREQATTR
   PE_SAML_UNKNOWN_ENTITY
   PE_SAML_SERVICE_NOT_ALLOWED
   PE_UNAUTHORIZEDPARTNER
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::SAML';
@@ -46,7 +48,8 @@ sub init {
     my $rule =
       $hd->buildSub( $hd->substitute( $self->conf->{issuerDBSAMLRule} ) );
     unless ($rule) {
-        $self->error( "Bad SAML rule -> " . $hd->tsv->{jail}->error );
+        my $error = $hd->tsv->{jail}->error || '???';
+        $self->error("Bad SAML activation rule -> $error");
         return 0;
     }
     $self->{rule} = $rule;
@@ -169,12 +172,15 @@ sub storeEnv {
     my ( $self, $req ) = @_;
     return PE_OK
       if ( $req->uri !~ $self->ssoUrlRe or $req->uri =~ $self->ssoUrlArtifact );
+
+# This doesn't always work, especially when coming from a HTTP-Redirect flow and
+# POST-ing the login form
     my ( $request, $response, $method, $relaystate, $artifact ) =
       $self->checkMessage( $req, $req->uri, $req->method, $req->content_type );
     return PE_OK if ( $artifact or !$request );
     my $login = $self->createLogin( $self->lassoServer );
     $self->disableSignatureVerification($login);
-    $self->processAuthnRequestMsg( $login, $request );
+    $self->processAuthnRequestMsg( $login, $request, 'debug' );
     if ( my $sp = $login->remote_providerID() ) {
         $req->env->{llng_saml_sp} = $sp;
         if ( my $spConfKey = $self->spList->{$sp}->{confKey} ) {
@@ -210,7 +216,7 @@ sub run {
     }
 
     # Session ID
-    my $session_id = $req->{sessionInfo}->{_session_id} || $req->{id};
+    my $session_id = $req->{sessionInfo}->{_session_id} || $req->id;
 
     # Session creation timestamp
     my $time = $req->{sessionInfo}->{_utime} || time();
@@ -383,15 +389,19 @@ sub run {
             $self->logger->debug("$sp match $spConfKey SP in configuration");
             $req->env->{llng_saml_spconfkey} = $spConfKey;
 
-            if ( my $rule = $self->spRules->{$sp} ) {
+            if ( my $rule = $self->spRules->{$spConfKey} ) {
                 unless ( $rule->( $req, $req->sessionInfo ) ) {
                     $self->userLogger->warn( 'User '
                           . $req->sessionInfo->{ $self->conf->{whatToTrace} }
-                          . "was not authorizated to access to $sp" );
+                          . " is not authorized to access to $sp" );
                     return PE_UNAUTHORIZEDPARTNER;
                 }
             }
 
+            $self->userLogger->notice( 'User '
+                  . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                  . " is authorized to access to $sp" );
+
             # Do we check signature?
             my $checkSSOMessageSignature =
               $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
@@ -605,14 +615,23 @@ sub run {
                 # Name is required
                 next unless $name;
 
-                # Error if corresponding attribute is not in user session
-                my $value = $req->{sessionInfo}->{$_};
+                # Lookup attribute value in SP macros or session
+                my $value;
+                if ( $self->spMacros->{$sp}->{$_} ) {
+                    $value = $self->spMacros->{$sp}->{$_}
+                      ->( $req, $req->{sessionInfo} );
+                }
+                else {
+                    $value = $req->{sessionInfo}->{$_};
+                }
+
+                # Check whether the value is required or not
                 unless ( defined $value ) {
                     if ($mandatory) {
                         $self->logger->error(
 "Session key $_ is required to set SAML $name attribute"
                         );
-                        return PE_SAML_SSO_ERROR;
+                        return PE_ISSUERMISSINGREQATTR;
                     }
                     else {
                         $self->logger->debug(
@@ -917,8 +936,10 @@ sub run {
                   . ' width="0" height="0" frameborder="0"></iframe>';
 
                 $req->info(
-                    $self->loadTemplate( 'simpleInfo',
-                        params => { trspan => 'updateCdc' } )
+                    $self->loadTemplate(
+                        $req, 'simpleInfo',
+                        params => { trspan => 'updateCdc' }
+                      )
                       . $cdc_iframe
                 );
             }
@@ -1252,7 +1273,7 @@ sub logout {
     return PE_OK if ( $req->data->{samlSLOCalled} );
 
     # Session ID
-    my $session_id = $req->{sessionInfo}->{_session_id} || $req->{id};
+    my $session_id = $req->{sessionInfo}->{_session_id} || $req->id;
 
     # Close SAML sessions
     unless ( $self->deleteSAMLSecondarySessions($session_id) ) {
@@ -1471,7 +1492,7 @@ sub sloRelayTerm {
     my $session = $logout->get_session();
 
     unless ($session) {
-        $self->lmLog( "Could not get session from logout", 'error' );
+        $self->logger->error("Could not get session from logout");
         return PE_SAML_SLO_ERROR;
     }
 
@@ -1590,7 +1611,7 @@ sub sloServer {
 
         if ($session_index) {
             my $sessionIndexSession = $self->getSamlSession($session_index);
-            return $self->p->sendError( $req, 'SAML session not found', 400 )
+            return $self->p->do( $req, [ sub { PE_SESSIONEXPIRED } ] )
               unless $sessionIndexSession;
 
             $local_session_id = $sessionIndexSession->data->{_saml_id};
@@ -1617,7 +1638,7 @@ sub sloServer {
 
             unless ($local_session) {
                 $self->logger->error("No local session found");
-                return $self->sendSLOErrorResponse( $logout, $method );
+                return $self->sendSLOErrorResponse( $req, $logout, $method );
             }
 
             # Load Session and Identity if they exist
@@ -1668,7 +1689,7 @@ sub sloServer {
         }
 
         # Check Destination
-        return $self->sendSLOErrorResponse( $logout, $method )
+        return $self->sendSLOErrorResponse( $req, $logout, $method )
           unless ( $self->checkDestination( $logout->request, $url ) );
 
         # Validate request if no previous error
@@ -1692,7 +1713,7 @@ sub sloServer {
 
         # Create SLO status session and get ID
         my $sloStatusSessionInfo = $self->getSamlSession( undef, $sloInfos );
-        my $relayID = $sloStatusSessionInfo->id;
+        my $relayID              = $sloStatusSessionInfo->id;
 
         $self->logger->debug("Create relay session $relayID");
 
@@ -1710,7 +1731,8 @@ sub sloServer {
         $req->data->{samlSLOCalled} = 1;
 
         # Launch normal logout and ignore errors
-        $self->p->do( $req, [ @{ $self->p->beforeLogout }, 'deleteSession' ] );
+        $req->steps( [ @{ $self->p->beforeLogout }, 'deleteSession' ] );
+        $self->p->process($req);
 
         # Signature
         my $signSLOMessage = $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
@@ -1718,7 +1740,7 @@ sub sloServer {
 
         unless ($signSLOMessage) {
             $self->logger->debug("Do not sign this SLO response");
-            return $self->sendSLOErrorResponse( $logout, $method )
+            return $self->sendSLOErrorResponse( $req, $logout, $method )
               unless ( $self->disableSignature($logout) );
         }
 
