@@ -12,7 +12,7 @@ use Unicode::String qw(utf8);
 use Scalar::Util 'weaken';
 use utf8;
 
-our $VERSION  = '2.0.3';
+our $VERSION  = '2.0.8';
 our $ppLoaded = 0;
 
 BEGIN {
@@ -183,7 +183,15 @@ sub userBind {
                 $self->{portal}->userLogger->warn("Bad password");
                 return PE_BADCREDENTIALS;
             }
-            return ( $mesg->code == 0 ? PE_OK : PE_LDAPERROR );
+            elsif ( $mesg->code == 0 ) {
+                return PE_OK;
+            }
+            else {
+                $self->{portal}->logger->error( "Bind failed with error "
+                      . $mesg->code . ": "
+                      . $mesg->error );
+                return PE_LDAPERROR;
+            }
         }
 
         # Check for ppolicy error
@@ -207,6 +215,9 @@ sub userBind {
 
             # Get expiration warning and graces
             if ( $resp->grace_authentications_remaining ) {
+                $self->{portal}->logger->debug(
+                    "LDAP password policy - grace authentications remaining: "
+                      . $resp->grace_authentications_remaining );
                 $req->info(
                     $self->{portal}->loadTemplate(
                         $req,
@@ -219,14 +230,20 @@ sub userBind {
             }
 
             if ( $resp->time_before_expiration ) {
+                $self->{portal}->logger->debug(
+                    "LDAP password policy - time before expiration: "
+                      . $resp->time_before_expiration );
                 $req->info(
                     $self->{portal}->loadTemplate(
                         $req,
                         'simpleInfo',
                         params => {
-                            trspan => 'authRemaining,'
-                              . $self->convertSec(
-                                $resp->time_before_expiration
+                            trspan => 'pwdWillExpire,'
+                              . join(
+                                ',',
+                                $self->convertSec(
+                                    $resp->time_before_expiration
+                                )
                               )
                         }
                     )
@@ -241,25 +258,29 @@ sub userBind {
         if ( $mesg->code == 0 ) {
             return PE_OK;
         }
+        else {
+            $req->data->{ldapError} = $mesg->error;
+        }
     }
     $self->{portal}->userLogger->warn("Bad password for $req->{user}");
     return PE_BADCREDENTIALS;
 }
 
-## @method int userModifyPassword(string dn, string newpassword, string oldpassword, boolean ad)
+## @method int userModifyPassword(string dn, string newpassword, string oldpassword, boolean ad, boolean requireOldPassword)
 # Change user's password.
 # @param $dn DN
 # @param $newpassword New password
 # @param $oldpassword Current password
 # @param $ad Active Directory mode
+# @param $requireOldPassword Old password is needed to update
 # @return Lemonldap::NG::Portal constant
 sub userModifyPassword {
-    my ( $self, $dn, $newpassword, $oldpassword, $ad ) = @_;
-    my $ppolicyControl     = $self->{conf}->{ldapPpolicyControl};
-    my $setPassword        = $self->{conf}->{ldapSetPassword};
-    my $asUser             = $self->{conf}->{ldapChangePasswordAsUser};
-    my $requireOldPassword = $self->{conf}->{portalRequireOldPassword};
-    my $passwordAttribute  = "userPassword";
+    my ( $self, $dn, $newpassword, $oldpassword, $ad, $requireOldPassword ) =
+      @_;
+    my $ppolicyControl    = $self->{conf}->{ldapPpolicyControl};
+    my $setPassword       = $self->{conf}->{ldapSetPassword};
+    my $asUser            = $self->{conf}->{ldapChangePasswordAsUser};
+    my $passwordAttribute = "userPassword";
     my $err;
     my $mesg;
 
@@ -379,14 +400,35 @@ sub userModifyPassword {
         }
         $self->{portal}
           ->logger->debug( 'Modification return code: ' . $mesg->code );
+        $self->{portal}
+          ->logger->debug( 'Modification return error: ' . $mesg->error );
+
+        # Manage specific errors for IBM Tivoli DS
+        if ( $self->{conf}->{ldapITDS} ) {
+            my $itds_code = $self->getITDSError($mesg);
+            return $itds_code unless ( $itds_code == PE_PASSWORD_OK );
+        }
+
+        # Manage specific errors for Active Directory
+        if ($ad) {
+            return PE_PP_INSUFFICIENT_PASSWORD_QUALITY
+              if ( $mesg->code == 53 );
+            return PE_PP_PASSWORD_MOD_NOT_ALLOWED
+              if ( $mesg->code == 19 );
+        }
+
+        # Standard errors
         return PE_WRONGMANAGERACCOUNT
           if ( $mesg->code == 50 || $mesg->code == 8 );
-        return PE_PP_INSUFFICIENT_PASSWORD_QUALITY
-          if ( $mesg->code == 53 && $ad );
-        return PE_PP_PASSWORD_MOD_NOT_ALLOWED
-          if ( $mesg->code == 19 && $ad );
-        return PE_LDAPERROR unless ( $mesg->code == 0 );
-        $self->{portal}->userLogger->notice("Password changed $dn");
+        unless ( $mesg->code == 0 ) {
+            $self->{portal}
+              ->logger->error( "Password modification failed with LDAP error "
+                  . $mesg->code . ": "
+                  . $mesg->error );
+            return PE_LDAPERROR;
+        }
+
+        $self->{portal}->userLogger->notice("Password changed for $dn");
 
         # Rebind as manager for next LDAP operations if we were bound as user
         $self->bind() if $asUser;
@@ -547,6 +589,9 @@ sub userModifyPassword {
             }
         }
         else {
+            $self->{portal}->logger->error(
+                "Missing PPolicy control from server response. Code: "
+                  . $mesg->code );
             return PE_LDAPERROR;
         }
     }
@@ -725,4 +770,30 @@ sub convertSec {
     return ( $day, $hrs, $min, $sec );
 }
 
+## @method int getITDSError(Net::LDAP::Message mesg)
+# Check error message to return according error code
+# @param mesg Modification return message
+# @return portal error code
+sub getITDSError {
+    my ( $self, $mesg ) = @_;
+
+    return PE_PP_MUST_SUPPLY_OLD_PASSWORD
+      if ( $mesg->code == 53 && $mesg->error =~ /Must supply old password/i );
+    return PE_PP_CHANGE_AFTER_RESET
+      if ( $mesg->code == 53
+        && $mesg->error =~ /Password must be changed after reset/i );
+    return PE_PP_PASSWORD_MOD_NOT_ALLOWED
+      if ( $mesg->code == 53
+        && $mesg->error =~ /Password may not be modified/i );
+    return PE_PP_PASSWORD_TOO_YOUNG
+      if ( $mesg->code == 19 && $mesg->error =~ /Password too young/i );
+    return PE_PP_PASSWORD_TOO_SHORT
+      if ( $mesg->code == 19 && $mesg->error =~ /Password too short/i );
+    return PE_PP_PASSWORD_IN_HISTORY
+      if ( $mesg->code == 19 && $mesg->error =~ /Password in History/i );
+    return PE_PP_INSUFFICIENT_PASSWORD_QUALITY if ( $mesg->code == 19 );
+
+    return PE_PASSWORD_OK;
+}
+
 1;
