@@ -3,8 +3,9 @@ package Lemonldap::NG::Portal::Lib::Notifications::JSON;
 use strict;
 use Mouse;
 use JSON qw(from_json);
+use POSIX qw(strftime);
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 no warnings 'redefine';
 
@@ -31,21 +32,28 @@ sub checkForNotifications {
     my $uid = $req->sessionInfo->{ $self->notifObject->notifField };
     my ( $notifs, $forUser ) = $self->notifObject->getNotifications($uid);
     my $form;
-    return 0 unless ($notifs);
+    unless ($notifs) {
+        $self->logger->info("No notification found");
+        return 0;
+    };
 
     # Transform notifications
-    my $i = 0;    #Files count
+    my $i = 0;    # Files count
     my @res;
+    my $now = strftime "%Y-%m-%d", localtime;
+
     foreach my $file ( values %$notifs ) {
-        my $json = from_json( $file, { allow_nonref => 1 } );
-        my $j = 0;    #Notifications count
+        my $json = eval { from_json( $file, { allow_nonref => 1 } ) };
+        $self->userLogger->warn(
+            "Bad JSON file: a notification for $uid was not done ($@)")
+          if ($@);
+        my $j = 0;    # Notifications count
         $json = [$json] unless ( ref $json eq 'ARRAY' );
       LOOP: foreach my $notif ( @{$json} ) {
 
             # Get the reference
             my $reference = $notif->{reference};
-
-            $self->logger->debug("Get reference $reference");
+            $self->logger->debug("Get reference: $reference");
 
             # Check it in session
             if ( exists $req->{sessionInfo}->{"notification_$reference"} ) {
@@ -55,6 +63,35 @@ sub checkForNotifications {
                     "Notification $reference was already accepted");
                 next LOOP;
             }
+
+            # Check date
+            my $date = $notif->{date};
+            $self->logger->debug("Get date: $date");
+            unless ( $date and $date =~ /\b\d{4}-\d{2}-\d{2}\b/ ) {
+                $self->logger->error('Malformed date');
+                next LOOP;
+            }
+            unless ( $date le $now ) {
+                $self->logger->debug('Notification date not reached');
+                next LOOP;
+            }
+
+            # Check condition if any
+            if ( my $condition = $notif->{condition} ) {
+                $self->logger->debug("Get condition $condition");
+                $condition = $self->p->HANDLER->substitute($condition);
+                unless ( $condition = $self->p->HANDLER->buildSub($condition) )
+                {
+                    $self->logger->error( 'Notification condition error: '
+                          . $self->p->HANDLER->tsv->{jail}->error );
+                    next LOOP;
+                }
+                unless ( $condition->( $req, $req->sessionInfo ) ) {
+                    $self->logger->debug(
+                        'Notification condition not authorized');
+                    next LOOP;
+                }
+            }
             push @res, $notif;
             $j++;
         }
@@ -70,6 +107,67 @@ sub checkForNotifications {
 
     # Stop here if nothing to display
     return 0 unless $i;
+    $self->userLogger->info("$i pending notification(s) found for $uid");
+
+    # Returns HTML fragment
+    return $form;
+}
+
+# Search for accepted notification and if any, returns HTML fragment.
+sub viewNotification {
+    my ( $self, $req, $ref, $epoch ) = @_;
+
+    # Look for accepted notifications in database
+    my $uid = $req->userData->{ $self->notifObject->notifField };
+    my ( $notifs, $forUser ) =
+      $self->notifObject->getAcceptedNotifs( $uid, $ref );
+    my $form;
+    unless ($notifs) {
+        $self->logger->info("No accepted notification found");
+        return 0;
+    };
+
+    # Transform notifications
+    my $i = 0;    # Files count
+    my @res;
+
+    foreach my $file ( values %$notifs ) {
+        my $json = eval { from_json( $file, { allow_nonref => 1 } ) };
+        $self->userLogger->warn(
+            "Bad JSON file: a notification for $uid was not done ($@)")
+          if ($@);
+        my $j = 0;    # Notifications count
+        $json = [$json] unless ( ref $json eq 'ARRAY' );
+      LOOP: foreach my $notif ( @{$json} ) {
+
+            # Get the reference
+            my $reference = $notif->{reference};
+            $self->logger->debug("Get reference: $reference");
+
+            # Check it in session
+            unless (exists $req->{userData}->{"notification_$reference"}
+                and $req->{userData}->{"notification_$reference"} eq $epoch
+                and $reference eq $ref )
+            {
+
+                # The notification is not already accepted
+                $self->logger->debug(
+                    "Notification $reference is not already accepted");
+                next LOOP;
+            }
+            push @res, $notif;
+            $j++;
+        }
+
+        # Go to next file if no notification found
+        next unless $j;
+        $i++;
+    }
+    $form .= $self->toForm( $req, @res );
+
+    # Stop here if nothing to display
+    return 0 unless $i;
+    $self->userLogger->info("$i accepted notification(s) found for $uid");
 
     # Returns HTML fragment
     return $form;
@@ -114,8 +212,9 @@ sub getNotifBack {
     $self->p->importHandlerData($req);
     my $uid = $req->sessionInfo->{ $self->notifObject->notifField };
 
-    my ( $notifs, $forUser );
-    eval { ( $notifs, $forUser ) = $self->notifObject->getNotifications($uid) };
+    # ALL notifications are returned here => Need to check active ones only
+    my ( $notifs, $forUser ) =
+      eval { $self->notifObject->getNotifications($uid) };
     return $self->p->sendError( $req, $@, 500 ) if ($@);
 
     if ($notifs) {
@@ -134,6 +233,7 @@ sub getNotifBack {
         }
 
         my $result = 1;
+        my $now    = strftime "%Y-%m-%d", localtime;
         foreach my $fileName ( keys %$notifs ) {
             my $file       = $notifs->{$fileName};
             my $fileResult = 1;
@@ -141,14 +241,48 @@ sub getNotifBack {
             $json = [$json] unless ( ref $json eq 'ARRAY' );
 
             # Get pending notifications and verify that they have been accepted
-            foreach my $notif (@$json) {
+          LOOP: foreach my $notif (@$json) {
                 my $reference = $notif->{reference};
 
+                # Check date
+                my $date = $notif->{date};
+                $self->logger->debug("Get date: $date");
+                unless ( $date and $date =~ /\b\d{4}-\d{2}-\d{2}\b/ ) {
+                    $self->logger->error('Malformed date');
+                    next LOOP;
+                }
+                unless ( $date le $now ) {
+                    $self->logger->debug('Notification date not reached');
+                    $fileResult = 0;    # Do not delete notification
+                    next LOOP;
+                }
+
+                # Check condition if any
+                if ( my $condition = $notif->{condition} ) {
+                    $self->logger->debug("Get condition $condition");
+                    $condition = $self->p->HANDLER->substitute($condition);
+                    unless ( $condition =
+                        $self->p->HANDLER->buildSub($condition) )
+                    {
+                        $self->logger->error( 'Notification condition error: '
+                              . $self->p->HANDLER->tsv->{jail}->error );
+                        next LOOP;
+                    }
+                    unless ( $condition->( $req, $req->sessionInfo ) ) {
+                        $self->logger->debug(
+                            'Notification condition not authorized');
+                        $fileResult = 0;    # Do not delete notification
+                        next LOOP;
+                    }
+                }
+
                 # Check if this pending notification has been seen
                 if ( my $refId = $refs->{$reference} ) {
 
                     # Verity that checkboxes have been checked
                     if ( $notif->{check} ) {
+                        $notif->{check} = [ $notif->{check} ]
+                          unless ( ref( $notif->{check} ) eq 'ARRAY' );
                         if ( my $toCheckCount = @{ $notif->{check} } ) {
                             unless ($checks->{$refId}
                                 and $toCheckCount == @{ $checks->{$refId} } )
@@ -157,7 +291,7 @@ sub getNotifBack {
 "$uid has not accepted notification $reference"
                                 );
                                 $result = $fileResult = 0;
-                                next;
+                                next LOOP;
                             }
                         }
                     }
@@ -165,10 +299,10 @@ sub getNotifBack {
                 else {
                     # Current pending notification has not been found in
                     # request
-                    $result = $fileResult = 0;
                     $self->logger->debug(
                         'Current pending notification has not been found');
-                    next;
+                    $result = $fileResult = 0;
+                    next LOOP;
                 }
 
                 # Register acceptation
@@ -217,6 +351,8 @@ sub toForm {
     @notifs = map {
         $i++;
         if ( $_->{check} ) {
+            $_->{check} = [ $_->{check} ]
+              unless ( ref( $_->{check} ) eq 'ARRAY' );
             my $j = 0;
             $_->{check} =
               [ map { $j++; { id => '1x' . $i . 'x' . $j, value => $_ } }
@@ -238,8 +374,10 @@ sub notificationServer {
     my ( $res, $err );
     if ( $req->method =~ /^POST$/i ) {
         $self->p->logger->debug("POST request");
-        ( $res, $err ) =
-          eval { $self->notifObject->newNotification( $req->content ) };
+        ( $res, $err ) = eval {
+            $self->notifObject->newNotification( $req->content,
+                $self->conf->{notificationDefaultCond} );
+        };
         return $self->p->sendError( $req, $@, 500 ) if ($@);
     }
     elsif ( $req->method =~ /^GET$/i ) {
@@ -252,10 +390,8 @@ sub notificationServer {
         $res = [];
         foreach my $notif ( keys %$notifs ) {
             $self->p->logger->debug("Found notification $notif");
-            my $json;
-            eval {
-                $json = from_json( $notifs->{$notif}, { allow_nonref => 1 } );
-            };
+            my $json =
+              eval { from_json( $notifs->{$notif}, { allow_nonref => 1 } ) };
             return $self->p->sendError( $req, "Unable to decode JSON file: $@",
                 400 )
               if ($@);
@@ -270,7 +406,10 @@ sub notificationServer {
             }
             else {
                 push @$res,
-                  { "uid" => $json->{uid}, "reference" => $json->{reference} };
+                  {
+                    "uid"       => $json->{uid},
+                    "reference" => ( $json->{reference} || $json->{ref} )
+                  };
             }
         }
     }
