@@ -5,10 +5,18 @@ use Mouse;
 use SOAP::Lite;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_ERROR PE_BADCREDENTIALS);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.8';
 
 # INITIALIZATION
 
+has urn => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub {
+        $_[0]->conf->{soapProxyUrn};
+    }
+);
+
 sub init {
     my ($self) = @_;
     $self->conf->{remoteCookieName}    ||= $self->conf->{cookieName};
@@ -29,10 +37,10 @@ no warnings 'once';
 
 sub getUser {
     my ( $self, $req ) = @_;
-    return PE_OK if ( $req->datas->{_proxyQueryDone} );
-    my $soap = SOAP::Lite->proxy( $self->conf->{proxyAuthService} )
-      ->uri('urn:Lemonldap/NG/Common/PSGI/SOAPService');
-    my $r = $soap->getCookies( $req->{user}, $req->datas->{password} );
+    return PE_OK if ( $req->data->{_proxyQueryDone} );
+    my $soap =
+      SOAP::Lite->proxy( $self->conf->{proxyAuthService} )->uri( $self->urn );
+    my $r = $soap->getCookies( $req->{user}, $req->data->{password} );
     if ( $r->fault ) {
         $self->logger->error( "Unable to query authentication service: "
               . $r->fault->{faultstring} );
@@ -47,22 +55,22 @@ sub getUser {
         $self->setSecurity($req);
         return PE_BADCREDENTIALS;
     }
-    unless ( $req->datas->{_remoteId} =
+    unless ( $req->data->{_remoteId} =
         $res->{cookies}->{ $self->conf->{remoteCookieName} } )
     {
         $self->logger->error("No cookie named $self->{remoteCookieName}");
         return PE_ERROR;
     }
-    $req->datas->{_proxyQueryDone}++;
+    $req->data->{_proxyQueryDone}++;
     PE_OK;
 }
 
 sub setSessionInfo {
     my ( $self, $req ) = @_;
-    return PE_OK if ( $req->datas->{_setSessionInfoDone} );
+    return PE_OK if ( $req->data->{_setSessionInfoDone} );
     my $soap = SOAP::Lite->proxy( $self->conf->{proxySessionService} )
-      ->uri('urn:Lemonldap/NG/Common/PSGI/SOAPService');
-    my $r = $soap->getAttributes( $req->datas->{_remoteId} );
+      ->uri( $self->urn );
+    my $r = $soap->getAttributes( $req->data->{_remoteId} );
     if ( $r->fault ) {
         $self->logger->error( "Unable to query authentication service"
               . $r->fault->{faultstring} );
@@ -76,7 +84,7 @@ sub setSessionInfo {
         $req->{sessionInfo}->{$_} ||= $res->{attributes}->{$_}
           unless (/^_/);
     }
-    $req->datas->{_setSessionInfoDone}++;
+    $req->data->{_setSessionInfoDone}++;
     PE_OK;
 }
 
