@@ -6,7 +6,7 @@ use strict;
 use Mouse;
 use Clone 'clone';
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -30,7 +30,8 @@ has menuModules => (
     }
 );
 
-has specific => ( is => 'rw', default => sub { {} } );
+has specific      => ( is => 'rw', default => sub { {} } );
+has sfManagerRule => ( is => 'rw', default => sub { 1 } );
 
 has imgPath => (
     is      => 'rw',
@@ -44,7 +45,13 @@ has imgPath => (
 # INITIALIZATION
 
 sub init {
-    1;
+    my ($self) = @_;
+
+    $self->sfManagerRule(
+        $self->p->buildRule( $self->conf->{sfManagerRule}, 'sfManagerRule' ) );
+    $self->sfManagerRule(1) unless $self->sfManagerRule;
+
+    return 1;
 }
 
 # RUNNING METHODS
@@ -60,7 +67,7 @@ sub params {
     $self->{conf}->{imgPath} ||= $self->{staticPrefix};
     my %res;
     my @defaultTabs = (qw/appslist password logout loginHistory oidcConsents/);
-    my @customTabs = split( /,\s*/, $self->{conf}->{customMenuTabs} || '' );
+    my @customTabs  = split( /,\s*/, $self->{conf}->{customMenuTabs} || '' );
 
     # Tab to display
     # Get the tab URL parameter
@@ -109,11 +116,43 @@ sub params {
     $res{AUTH_ERROR_TYPE} =
       $req->error_type( $res{AUTH_ERROR} = $req->menuError );
 
-    # Display menu 2fRegisters link only if at least a 2F device is registered
+# Display menu 2fRegisters link only if at least a 2F device is registered and rule
     $res{sfaManager} =
-      $self->p->_sfEngine->display2fRegisters( $req, $req->userData );
+         $self->p->_sfEngine->display2fRegisters( $req, $req->userData )
+      && $self->sfManagerRule->( $req, $req->userData );
     $self->logger->debug("Display 2fRegisters link") if $res{sfaManager};
 
+    # Display refresh my rights unless disabled
+    $res{RefreshMyRights} = $self->conf->{portalDisplayRefreshMyRights};
+
+    # Display menu links only if required
+    foreach (qw(ContextSwitching DecryptValue Notifications)) {
+        my $plugin =
+          $self->p->loadedModules->{"Lemonldap::NG::Portal::Plugins::$_"};
+        $res{$_} =
+            $plugin
+          ? $plugin->displayLink( $req, $req->userData )
+          : '';
+        my $msg = "Display $_ link";
+        $msg .= " -> $res{ContextSwitching}"
+          if ( $_ eq 'ContextSwitching' && $res{$_} );
+        $self->logger->debug($msg) if $res{$_};
+        undef $plugin;
+    }
+
+    # Decide whether to display the dropdown or regular text
+    $res{DropdownMenu} = 0;
+
+    foreach (
+        qw(RefreshMyRights sfaManager Notifications DecryptValue ContextSwitching)
+      )
+    {
+        if ( $res{$_} ) {
+            $res{DropdownMenu} = 1;
+            last;
+        }
+    }
+
     return %res;
 }
 
@@ -136,17 +175,17 @@ sub displayModules {
             }
             elsif ( $module->[0] eq 'LoginHistory' ) {
                 $moduleHash->{'SUCCESS_LOGIN'} =
-                  $self->p->mkSessionArray(
+                  $self->p->mkSessionArray( $req,
                     $req->{sessionInfo}->{_loginHistory}->{successLogin},
                     "", 0, 0 );
                 $moduleHash->{'FAILED_LOGIN'} =
-                  $self->p->mkSessionArray(
+                  $self->p->mkSessionArray( $req,
                     $req->{sessionInfo}->{_loginHistory}->{failedLogin},
                     "", 0, 1 );
             }
             elsif ( $module->[0] eq 'OidcConsents' ) {
                 $moduleHash->{'OIDC_CONSENTS'} =
-                  $self->p->mkOidcConsent( $req->sessionInfo );
+                  $self->p->mkOidcConsent( $req, $req->sessionInfo );
             }
             push @$displayModules, $moduleHash;
         }
@@ -168,7 +207,7 @@ sub appslist {
     my $catlevel = 0;
 
     my $applicationList = clone( $self->conf->{applicationList} );
-    my $filteredList = $self->_filter( $req, $applicationList );
+    my $filteredList    = $self->_filter( $req, $applicationList );
     push @$appslist,
       $self->_buildCategoryHash( $req, "", $filteredList, $catlevel );
 
@@ -193,7 +232,7 @@ sub _buildCategoryHash {
     # Extract applications from hash
     my $apphash;
     foreach my $catkey ( sort keys %$cathash ) {
-        next if $catkey =~ /(type|options|catname)/;
+        next if $catkey =~ /(type|options|catname|order)/;
         if ( $cathash->{$catkey}->{type} eq "application" ) {
             $apphash->{$catkey} = $cathash->{$catkey};
         }
@@ -201,15 +240,31 @@ sub _buildCategoryHash {
 
     # Display applications first
     if ( scalar keys %$apphash > 0 ) {
-        foreach my $appkey ( sort keys %$apphash ) {
+        foreach my $appkey (
+            sort {
+                ( $apphash->{$a}->{order} || 0 )
+                  <=> ( $apphash->{$b}->{order} || 0 )
+                  or $a cmp $b
+            }
+            keys %$apphash
+          )
+        {
             push @$applications,
               $self->_buildApplicationHash( $appkey, $apphash->{$appkey} );
         }
     }
 
     # Display subcategories
-    foreach my $catkey ( sort keys %$cathash ) {
-        next if $catkey =~ /(type|options|catname)/;
+    foreach my $catkey (
+        sort {
+            ( $cathash->{$a}->{order} || 0 )
+              <=> ( $cathash->{$b}->{order} || 0 )
+              or $a cmp $b
+        }
+        grep { not /^(?:catname|type|options|order)$/ } keys %$cathash
+      )
+    {
+
         if ( $cathash->{$catkey}->{type} eq "category" ) {
             push @$categories,
               $self->_buildCategoryHash( $req, $catkey, $cathash->{$catkey},
@@ -238,15 +293,16 @@ sub _buildApplicationHash {
     my $applications;
 
     # Get application items
-    my $appname = $apphash->{options}->{name} || $appid;
-    my $appuri  = $apphash->{options}->{uri}  || "";
+    my $appname = $apphash->{options}->{name}    || $appid;
+    my $appuri  = $apphash->{options}->{uri}     || "";
     my $appdesc = $apphash->{options}->{description};
     my $applogo = $apphash->{options}->{logo};
+    my $apptip  = $apphash->{options}->{tooltip} || $appname;
 
     # Detect sub applications
     my $subapphash;
     foreach my $key ( sort keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
+        next if $key =~ /(type|options|catname|order)/;
         if ( $apphash->{$key}->{type} eq "application" ) {
             $subapphash->{$key} = $apphash->{$key};
         }
@@ -254,7 +310,15 @@ sub _buildApplicationHash {
 
     # Display sub applications
     if ( scalar keys %$subapphash > 0 ) {
-        foreach my $appkey ( sort keys %$subapphash ) {
+        foreach my $appkey (
+            sort {
+                ( $subapphash->{$a}->{order} || 0 )
+                  <=> ( $subapphash->{$b}->{order} || 0 )
+                  or $a cmp $b
+            }
+            keys %$subapphash
+          )
+        {
             push @$applications,
               $self->_buildApplicationHash( $appkey, $subapphash->{$appkey} );
         }
@@ -267,6 +331,7 @@ sub _buildApplicationHash {
         appdesc     => $appdesc,
         applogo     => $applogo,
         appid       => $appid,
+        apptip      => $apptip,
     };
     $applicationHash->{applications} = $applications if $applications;
     return $applicationHash;
@@ -305,7 +370,7 @@ sub _filterHash {
     my ( $self, $req, $apphash ) = @_;
 
     foreach my $key ( keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
+        next if $key =~ /(type|options|catname|order)/;
         if (    $apphash->{$key}->{type}
             and $apphash->{$key}->{type} eq "category" )
         {
@@ -319,7 +384,7 @@ sub _filterHash {
 
             # Find sub applications and filter them
             foreach my $appkey ( keys %{ $apphash->{$key} } ) {
-                next if $appkey =~ /(type|options|catname)/;
+                next if $appkey =~ /(type|options|catname|order)/;
 
                 # We have sub elements, so we filter them
                 $self->_filterHash( $req, $apphash->{$key} );
@@ -393,7 +458,7 @@ sub _isCategoryEmpty {
 
     # Test sub categories
     foreach $key ( keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
+        next if $key =~ /(type|options|catname|order)/;
         if (    $apphash->{$key}->{type}
             and $apphash->{$key}->{type} eq "category" )
         {
@@ -408,10 +473,12 @@ sub _isCategoryEmpty {
         # Temporary store 'options'
         my $tmp_options = $apphash->{options};
         my $tmp_catname = $apphash->{catname};
+        my $tmp_order   = $apphash->{order};
 
         delete $apphash->{type};
         delete $apphash->{options};
         delete $apphash->{catname};
+        delete $apphash->{order};
 
         if ( scalar( keys %$apphash ) ) {
 
@@ -420,6 +487,7 @@ sub _isCategoryEmpty {
             $apphash->{type}    = "category";
             $apphash->{options} = $tmp_options;
             $apphash->{catname} = $tmp_catname;
+            $apphash->{order}   = $tmp_order;
 
             # Return false
             return 0;
