@@ -1,6 +1,6 @@
 package Lemonldap::NG::Portal::Main::Process;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -39,12 +39,19 @@ sub process {
             }
         }
     }
-    $self->logger->debug(
-        "Returned error: $err (" . portalConsts->{$err} . ")" )
+    $self->logger->debug( "Returned " . $self->_formatProcessResult($err) )
       if ($err);
     return $err;
 }
 
+sub _formatProcessResult {
+    my ( $self, $err ) = @_;
+    return ( ( $err > 0 ? "error" : "status" )
+        . ": $err ("
+          . portalConsts->{$err}
+          . ")" );
+}
+
 # First process block: check args
 # -------------------------------
 
@@ -52,7 +59,7 @@ sub process {
 sub restoreArgs {
     my ( $self, $req ) = @_;
     $req->mustRedirect(1);
-    return PE_OK;
+    PE_OK;
 }
 
 sub importHandlerData {
@@ -95,12 +102,18 @@ sub controlUrl {
         }
         else {
             if ( $url =~ m#[^A-Za-z0-9\+/=]# ) {
-                $self->userLogger->error(
-                    "Value must be BASE64 encoded (param: url | value: $url)");
-                return PE_BADURL;
+                unless ( $req->maybeNotBase64 ) {
+                    $self->userLogger->error(
+"Value must be BASE64 encoded (param: url | value: $url)"
+                    );
+                    return PE_BADURL;
+                }
+                $req->{urldc} = $url;
+            }
+            else {
+                $req->{urldc} = decode_base64($url);
+                $req->{urldc} =~ s/[\r\n]//sg;
             }
-            $req->{urldc} = decode_base64($url);
-            $req->{urldc} =~ s/[\r\n]//sg;
         }
 
         # For logout request, test if Referer comes from an authorized site
@@ -123,21 +136,30 @@ sub controlUrl {
         }
 
         # Unprotected hosts
-        if ( $tmp and !$self->isTrustedUrl($tmp) ) {
+        my ( $vhost, $appuri ) = $tmp =~ m#^https?://([^/]*)(.*)#;
+        $vhost =~ s/:\d+$//;
+        $vhost = 'http://' . $self->HANDLER->resolveAlias($vhost);
+        $self->logger->debug( "Required URL (param: "
+              . ( $req->param('logout') ? 'HTTP Referer' : 'urldc' )
+              . " | value: $tmp | alias: $vhost)" );
+
+        if (    $tmp
+            and !$self->isTrustedUrl($tmp)
+            and !$self->isTrustedUrl($vhost) )
+        {
             $self->userLogger->error(
                     "URL contains a non protected host (param: "
                   . ( $req->param('logout') ? 'HTTP Referer' : 'urldc' )
-                  . " | value: $tmp)" );
+                  . " | value: $tmp | alias: $vhost)" );
             delete $req->{urldc};
             return PE_BADURL;
         }
 
-        $req->env->{urldc}  = $req->{urldc};
-        $req->env->{_url}   = $req->{_url};
-        $req->data->{_url}  = $url;
-        $req->pdata->{_url} = $url;
+        $req->env->{urldc} = $req->{urldc};
+        $req->env->{_url}  = $req->{_url};
+        $req->data->{_url} = $req->pdata->{_url} =
+          encode_base64( $req->{urldc}, '' );    # Avoid \n or \r
     }
-
     PE_OK;
 }
 
@@ -153,10 +175,12 @@ sub checkLogout {
 sub authLogout {
     my ( $self, $req ) = @_;
     my $res = $self->_authentication->authLogout($req);
-    unless ( $res or $req->pdata->{keepPdata} ) {
-        $self->logger->debug('Cleaning pdata');
-        $req->pdata( {} );
+    $self->logger->debug('Cleaning pdata');
+    my $tmp = $req->pdata->{keepPdata} //= [];
+    foreach my $k ( keys %{ $req->pdata } ) {
+        delete $req->pdata->{$k} unless ( grep { $_ eq $k } @$tmp );
     }
+    $req->pdata->{keepPdata} = $tmp if @$tmp;
     return $res;
 }
 
@@ -178,7 +202,20 @@ sub deleteSession {
         }
     }
 
-    # TODO
+    # Merge logoutServices from user context (for example CAS logoutServices
+    # url) and from global configuration
+    if ( $self->conf->{logoutServices}
+        and %{ $self->conf->{logoutServices} } )
+    {
+
+        # Initialize logoutServices (if not already done)
+        $req->data->{logoutServices} ||= {};
+        $req->data->{logoutServices} = {
+            %{ $req->data->{logoutServices} },
+            %{ $self->conf->{logoutServices} }
+        };
+    }
+
     # Collect logout services and build hidden iFrames
     if ( $req->data->{logoutServices} and %{ $req->data->{logoutServices} } ) {
 
@@ -192,15 +229,12 @@ sub deleteSession {
         );
 
         foreach ( keys %{ $req->data->{logoutServices} } ) {
-            my $logoutServiceName = $_;
-            my $logoutServiceUrl =
-              $req->data->{logoutServices}->{$logoutServiceName};
+            my $logoutServiceUrl = $req->data->{logoutServices}->{$_};
 
-            $self->logger->debug(
-                "Find logout service $logoutServiceName ($logoutServiceUrl)");
+            $self->logger->debug("Find logout service $_ ($logoutServiceUrl)");
 
             my $iframe =
-                qq'<iframe src="$logoutServiceUrl" alt="$logoutServiceName"'
+                qq'<iframe src="$logoutServiceUrl" alt="$_"'
               . ' marginwidth="0" marginheight="0" scrolling="no"'
               . ' class="hiddenFrame" width="0" height="0"'
               . ' frameborder="0"></iframe>';
@@ -255,7 +289,6 @@ sub checkXSSAttack {
             "XSS attack detected (param: $name | value: $value)");
         return $self->conf->{checkXSS};
     }
-
     return 0;
 }
 
@@ -316,7 +349,6 @@ sub authenticate {
     # Ignore result, process will end at least with PE_BADCREDENTIALS
     my $tmp = $self->process($req);
     $ret = $tmp if ( $tmp == PE_WAIT );
-
     return $ret;
 }
 
@@ -338,7 +370,7 @@ sub setSessionInfo {
     my ( $self, $req ) = @_;
 
     # Set _user
-    $req->{sessionInfo}->{_user} //= $req->{user};
+    $req->{sessionInfo}->{_user} //= $req->user;
 
     # Get the current user module
     $req->{sessionInfo}->{_auth}   = $self->getModule( $req, "auth" );
@@ -371,7 +403,6 @@ sub setSessionInfo {
 
     # Call UserDB setSessionInfo
     return $self->_userDB->setSessionInfo($req);
-
     PE_OK;
 }
 
@@ -393,7 +424,7 @@ sub setPersistentSessionInfo {
     my ( $self, $req ) = @_;
 
     # Do not restore infos if session already opened
-    unless ( $req->{id} ) {
+    unless ( $req->id ) {
         my $key = $req->{sessionInfo}->{ $self->conf->{whatToTrace} };
 
         return PE_OK unless ( $key and length($key) );
@@ -439,7 +470,7 @@ sub store {
     $req->userData( $req->sessionInfo );
 
     # Create second session for unsecure cookie
-    if ( $self->conf->{securedCookie} == 2 and !$req->refresh() ) {
+    if ( $self->conf->{securedCookie} == 2 and !$req->refresh ) {
         my %infos = %{ $req->{sessionInfo} };
         $infos{_updateTime} = strftime( "%Y%m%d%H%M%S", localtime() );
         $self->logger->debug("Set _updateTime with $infos{_updateTime}");
@@ -468,22 +499,29 @@ sub store {
 
     # Main session
     my $session = $self->getApacheSession(
-        $req->{id},
+        $req->id,
         force => $req->{force},
         info  => $infos
     );
     return PE_APACHESESSIONERROR unless ($session);
-    $req->id( $session->{id} );
+
+    # Update current request
+    $req->id( $session->id );
+    unless ( $self->_sfEngine->searchForAuthorized2Fmodules($req) ) {
+        $self->logger->debug(
+            "No 2F module authorized -> Update current request");
+        $req->{sessionInfo}->{_session_id}   = $session->{id};
+        $req->{sessionInfo}->{_session_kind} = $session->{kind};
+    }
 
     # Compute unsecured cookie value if needed
-    if ( $self->conf->{securedCookie} == 3 and !$req->refresh() ) {
+    if ( $self->conf->{securedCookie} == 3 and !$req->refresh ) {
         $req->{sessionInfo}->{_httpSession} =
-          $self->conf->{cipher}->encryptHex( $req->{id}, "http" );
+          $self->conf->{cipher}->encryptHex( $req->id, "http" );
         $self->logger->debug( " -> Compute unsecured cookie value : "
               . $req->{sessionInfo}->{_httpSession} );
     }
     $req->refresh(0);
-
     PE_OK;
 }
 
@@ -493,7 +531,7 @@ sub buildCookie {
         $req->addCookie(
             $self->cookie(
                 name   => $self->conf->{cookieName},
-                value  => $req->{id},
+                value  => $req->id,
                 domain => $self->conf->{domain},
                 secure => $self->conf->{securedCookie},
             )
@@ -509,8 +547,14 @@ sub buildCookie {
             );
         }
     }
-    $self->userLogger->notice(
-"User $req->{user} successfully authenticated at level $req->{sessionInfo}->{authenticationLevel}"
+    my $ref = (
+        %{ $req->{sessionInfo} }
+        ? $req->{sessionInfo}
+        : $req->{userData}
+    );
+    $self->userLogger->notice( 'User '
+          . $ref->{ $self->conf->{whatToTrace} }
+          . " successfully authenticated at level $ref->{authenticationLevel}"
     );
     PE_OK;
 }
