@@ -9,7 +9,7 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.5';
+our $VERSION = '2.0.8';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -19,12 +19,22 @@ use JSON;
 
 has trOverCache => ( is => 'rw', default => sub { {} } );
 
+# The execution order between groups and macros can be
+# modified in config (#1877)
+sub groupsAndMacros {
+    return (
+        $_[0]->conf->{groupsBeforeMacros}
+        ? qw(setGroups setMacros)
+        : qw(setMacros setGroups)
+    );
+}
+
 # List constants
 sub authProcess { qw(extractFormInfo getUser authenticate) }
 
 sub sessionData {
-    qw(setAuthSessionInfo setSessionInfo setMacros setGroups setPersistentSessionInfo
-      setLocalGroups store secondFactor);
+    return qw(setAuthSessionInfo setSessionInfo), $_[0]->groupsAndMacros,
+      qw(setPersistentSessionInfo setLocalGroups store secondFactor);
 }
 
 sub validSession {
@@ -52,9 +62,30 @@ sub handler {
             $self->logger->error("Bad JSON content in cookie pdata");
             $req->pdata( {} );
         }
+
+        # Avoid fatal errors when using old keepPdata format
+        if ( $req->pdata->{keepPdata}
+            and not( ref $req->pdata->{keepPdata} eq "ARRAY" ) )
+        {
+            $req->pdata->{keepPdata} = [];
+        }
     }
     my $res = $self->Lemonldap::NG::Common::PSGI::Router::handler($req);
 
+    # Avoid permanent loop 'Portal <-> _url' if pdata cookie is not removed
+    if (    $req->userData->{_url}
+        and !$req->pdata->{keepPdata}
+        and $req->userData->{_session_id}
+        and $req->{env}->{HTTP_COOKIE}
+        and $req->{env}->{HTTP_COOKIE} eq
+        encode_base64( $req->userData->{_url}, '' ) )
+    {
+        $self->logger->info("Force cleaning pdata");
+        $self->logger->warn("pdata cookie domain must be set")
+          unless ( $self->conf->{pdataDomain} );
+        $req->pdata( {} );
+    }
+
     # Save pdata
     if ( $sp or %{ $req->pdata } ) {
         my %v = (
@@ -62,11 +93,13 @@ sub handler {
             (
                 %{ $req->pdata }
                 ? ( value => uri_escape( JSON::to_json( $req->pdata ) ) )
-                : (
-                    value   => '',
-                    expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
-                )
-            )
+                : ( value => '', expires => 'Wed, 21 Oct 2015 00:00:00 GMT' )
+            ),
+            (
+                $self->conf->{pdataDomain}
+                ? ( domain => $self->conf->{pdataDomain}, )
+                : ()
+            ),
         );
         push @{ $res->[1] }, 'Set-Cookie', $self->cookie(%v);
     }
@@ -146,10 +179,12 @@ sub postAuthenticatedRequest {
 sub refresh {
     my ( $self, $req ) = @_;
     $req->mustRedirect(1);
-    my %data = %{ $req->userData };
-    $req->user( $data{ $self->conf->{whatToTrace} } );
+    my %data          = %{ $req->userData };
+    my $lastAuthLevel = $req->userData->{authenticationLevel};
+    $self->userLogger->notice(
+        'Refresh request for ' . $data{ $self->conf->{whatToTrace} } );
+    $req->user( $data{_user} || $data{ $self->conf->{whatToTrace} } );
     $req->id( $data{_session_id} );
-    $self->userLogger->notice( 'Refresh request for ' . $req->user );
     foreach ( keys %data ) {
         delete $data{$_} unless ( /^_/ or /^(?:startTime)$/ );
     }
@@ -161,8 +196,12 @@ sub refresh {
             @{ $self->betweenAuthAndData },
             'setAuthSessionInfo',
             'setSessionInfo',
-            'setMacros',
-            'setGroups',
+            sub {
+                $req->sessionInfo->{authenticationLevel} = $lastAuthLevel
+                  ;    # Restore previous authentication level (#2179)
+                return PE_OK;
+            },
+            $self->groupsAndMacros,
             'setLocalGroups',
             sub {
                 $req->sessionInfo->{$_} = $data{$_} foreach ( keys %data );
@@ -220,18 +259,32 @@ sub do {
     }
 
     # Remove userData if authentication fails
-    if ( $err == PE_BADCREDENTIALS ) {
+    if ( $err == PE_BADCREDENTIALS or $err == PE_BADOTP ) {
         $req->userData( {} );
     }
 
     if ( !$self->conf->{noAjaxHook} and $req->wantJSON ) {
         $self->logger->debug('Processing to JSON response');
         if ( ( $err > 0 and !$req->id ) or $err eq PE_SESSIONNOTGRANTED ) {
-            return [
-                401,
-                [ 'WWW-Authenticate' => "SSO " . $self->conf->{portal} ],
-                [qq'{"result":0,"error":$err}']
-            ];
+            my $json = { result => 0, error => $err };
+            if ( $req->wantErrorRender ) {
+                $json->{html} = $self->loadTemplate(
+                    $req,
+                    'errormsg',
+                    params => {
+                        AUTH_ERROR      => $err,
+                        AUTH_ERROR_TYPE => $req->error_type,
+                    }
+                );
+            }
+            return $self->sendJSONresponse(
+                $req, $json,
+                code    => 401,
+                headers => [
+                    'WWW-Authenticate' => "SSO " . $self->conf->{portal},
+                    "Content-Type"     => "application/javascript"
+                ],
+            );
         }
         elsif ( $err > 0 and $err != PE_PASSWORD_OK and $err != PE_LOGOUT_OK ) {
             return $self->sendJSONresponse(
@@ -246,7 +299,12 @@ sub do {
                 {
                     result => 1,
                     error  => $err,
-                    id     => $req->id
+                    id     => $req->id,
+                    (
+                        $req->sessionInfo->{_httpSession}
+                        ? ( id_http => $req->sessionInfo->{_httpSession} )
+                        : ()
+                    )
                 }
             );
         }
@@ -323,8 +381,14 @@ sub autoRedirect {
             $req->data->{redirectFormMethod} = "get";
         }
         else {
-            return [ 302,
-                [ Location => $req->{urldc}, @{ $req->respHeaders } ], [] ];
+            if (    $req->{pdata}->{_url}
+                and $req->{pdata}->{_url} eq encode_base64( $req->{urldc}, '' )
+              )
+            {
+                $self->logger->info("Force cleaning pdata");
+                $req->pdata( {} );
+            }
+            return [ 302, [ Location => $req->{urldc}, $req->spliceHdrs ], [] ];
         }
     }
     my ( $tpl, $prms ) = $self->display($req);
@@ -404,7 +468,7 @@ sub getApacheSession {
 sub getPersistentSession {
     my ( $self, $uid, $info ) = @_;
 
-    return unless defined $uid;
+    return unless ( defined $uid and !$self->conf->{disablePersistentStorage} );
 
     # Compute persistent identifier
     my $pid = $self->_md5hash($uid);
@@ -451,7 +515,11 @@ sub updatePersistentSession {
     my ( $self, $req, $infos, $uid, $id ) = @_;
 
     # Return if no infos to update
-    return () unless ( ref $infos eq 'HASH' and %$infos );
+    return ()
+      unless ( ref $infos eq 'HASH'
+        and %$infos
+        and !$self->conf->{disablePersistentStorage} );
+
     $uid ||= $req->{sessionInfo}->{ $self->conf->{whatToTrace} }
       || $req->userData->{ $self->conf->{whatToTrace} };
     $self->logger->debug("Found 'whatToTrace' -> $uid");
@@ -488,13 +556,14 @@ sub updateSession {
     return () unless ( ref $infos eq 'HASH' and %$infos );
 
     # Recover session ID unless given
-    $id ||= $req->{id} || $req->userData->{_session_id};
+    $id ||= $req->id || $req->userData->{_session_id};
 
     if ($id) {
 
         # Update sessionInfo data
         ## sessionInfo updated if $id defined : quite strange !!
         ## See https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues/430
+        $self->logger->debug("Update session $id");
         foreach ( keys %$infos ) {
             $self->logger->debug("Update sessionInfo $_");
             $self->_dump( $infos->{$_} );
@@ -615,7 +684,7 @@ sub autoPost {
 
     # Display info before redirecting
     if ( $req->info() ) {
-        $req->{infoFormMethod} = $req->param('method') || "post";
+        $req->data->{infoFormMethod} = $req->param('method') || "post";
         return PE_INFO;
     }
 
@@ -727,7 +796,9 @@ sub cookie {
     $h{HttpOnly} //= $self->conf->{httpOnly};
     $h{max_age}  //= $self->conf->{cookieExpiration}
       if ( $self->conf->{cookieExpiration} );
-    foreach (qw(domain path expires max_age HttpOnly)) {
+    $h{SameSite} ||= $self->conf->{sameSite};
+
+    foreach (qw(domain path expires max_age HttpOnly SameSite)) {
         my $f = $_;
         $f =~ s/_/-/g;
         push @res, "$f=$h{$_}" if ( $h{$_} );
@@ -738,10 +809,12 @@ sub cookie {
 
 sub _dump {
     my ( $self, $variable ) = @_;
-    require Data::Dumper;
-    $Data::Dumper::Indent  = 0;
-    $Data::Dumper::Useperl = 1;
-    $self->logger->debug( "Dump: " . Data::Dumper::Dumper($variable) );
+    if ( $self->conf->{logLevel} eq 'debug' ) {
+        require Data::Dumper;
+        $Data::Dumper::Indent  = 0;
+        $Data::Dumper::Useperl = 1;
+        $self->logger->debug( "Dump: " . Data::Dumper::Dumper($variable) );
+    }
     return;
 }
 
@@ -793,11 +866,11 @@ sub sendHtml {
       'Pragma'        => 'no-cache',                               # HTTP 1.0
       'Expires'       => '0';                                      # Proxies
 
-    my @cors = split /;/, $self->cors;
     if ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
         push @{ $res->[1] }, @cors;
-        $self->logger->debug(
-            "Apply following CORS policy : " . Data::Dumper::Dumper( \@cors ) );
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
     }
 
     # Set authorized URL for POST
@@ -835,10 +908,18 @@ sub sendHtml {
     $csp .= ';';
 
     # Deny using portal in frame except if it is required
-    unless ( $req->frame or $self->conf->{portalAntiFrame} == 0 ) {
+    unless ( $req->frame
+        or $self->conf->{portalAntiFrame} == 0
+        or $self->conf->{cspFrameAncestors} )
+    {
         push @{ $res->[1] }, 'X-Frame-Options' => 'DENY';
         $csp .= "frame-ancestors 'none';";
     }
+    if ( $self->conf->{cspFrameAncestors} ) {
+        push @{ $res->[1] }, 'X-Frame-Options' => 'ALLOW-FROM '
+          . "$self->{conf}->{cspFrameAncestors};";
+        $csp .= "frame-ancestors $self->{conf}->{cspFrameAncestors};";
+    }
 
     # Check if frames need to be embedded
     my @url;
@@ -941,6 +1022,7 @@ sub tplParams {
         PORTAL_URL => $self->conf->{portal},
         SKIN_PATH  => $portalPath . "skins",
         SKIN_BG    => $self->conf->{portalSkinBackground},
+        CUSTOM_CSS => $self->conf->{portalCustomCss},
         ( $self->customParameters ? ( %{ $self->customParameters } ) : () ),
         %templateParams
     );
@@ -1022,6 +1104,42 @@ sub _sumUpSession {
     return $res;
 }
 
+sub corsPreflight {
+    my ( $self, $req ) = @_;
+    my @headers;
+    if ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
+        push @headers, @cors;
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
+    }
+    return [ 204, \@headers, [] ];
+}
+
+sub sendJSONresponse {
+    my ( $self, $req, $j, %args ) = @_;
+    my $res = Lemonldap::NG::Common::PSGI::sendJSONresponse(@_);
+
+    # If this is a cross-domain request from the portal itself
+    # (Ajax SSL to a different VHost)
+    # we allow CORS
+    if ( $req->origin and index( $self->conf->{portal}, $req->origin ) == 0 ) {
+        $self->logger->debug('AJAX request from portal, allowing CORS');
+        push @{ $res->[1] },
+          "Access-Control-Allow-Origin"      => $req->origin,
+          "Access-Control-Allow-Methods"     => "*",
+          "Access-Control-Allow-Credentials" => "true";
+
+    }
+    elsif ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
+        push @{ $res->[1] }, @cors;
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
+    }
+    return $res;
+}
+
 # Temlate loader
 sub loadTemplate {
     my ( $self, $req, $name, %prm ) = @_;
@@ -1036,7 +1154,7 @@ sub loadTemplate {
         search_path_on_include => 1,
         die_on_bad_params      => 0,
         die_on_missing_include => 1,
-        cache                  => 1,
+        cache                  => ( defined $prm{cache} ? $prm{cache} : 1 ),
         global_vars            => 0,
         ( $prm{filter} ? ( filter => $prm{filter} ) : () ),
     );
