@@ -4,7 +4,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_INFO PE_OK);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
   'Lemonldap::NG::Portal::Lib::OtherSessions';
@@ -19,12 +19,12 @@ sub init { 1 }
 
 sub run {
     my ( $self, $req ) = @_;
+    
     if ( $req->param('checkLogins') ) {
         $self->logger->debug('History asked');
-        $req->info(
-            (
+        $req->info( (
                 $req->sessionInfo->{_loginHistory}->{successLogin}
-                ? $self->p->mkSessionArray(
+                ? $self->p->mkSessionArray( $req,
                     $req->sessionInfo->{_loginHistory}->{successLogin},
                     'lastLogins', 0, 0 )
                 : ""
@@ -32,15 +32,14 @@ sub run {
             . ("<hr>")
               . (
                 $req->sessionInfo->{_loginHistory}->{failedLogin}
-                ? $self->p->mkSessionArray(
+                ? $self->p->mkSessionArray( $req,
                     $req->sessionInfo->{_loginHistory}->{failedLogin},
                     'lastFailedLogins', 0, 1 )
                 : ""
               )
         );
-        unless ( $req->info ) {
-            $req->info( $self->loadTemplate('noHistory') );
-        }
+        $req->info( $self->loadTemplate( $req, 'noHistory' ) )
+          unless ( $req->info );
         return PE_INFO;
     }
     return PE_OK;
