@@ -6,6 +6,8 @@
 #   * GET /sessions/<type>/<session-id>/<key>    : get a session key value
 #   * GET /sessions/<type>/<session-id>/[k1,k2]  : get some session key value
 #   * POST /sessions/<type>                      : create a session
+#   * POST /sessions/<type>?all=1                : get all sessions (needs a token)
+#   * POST /sessions/<type>?all=1&search=uid,dwho: search all sessions where uid=dwho (needs a token)
 #   * PUT /sessions/<type>/<session-id>          : update some keys
 #   * DELETE /sessions/<type>/<session-id>       : delete a session
 #
@@ -13,8 +15,10 @@
 #   * GET /session/my/<type>                     : get session data
 #   * GET /session/my/<type>/key                 : get session key
 #   * DELETE /session/my                         : ask for logout
+#   * DELETE /sessions/my                        : ask for global logout
 #
 # - Authentication
+#   * GET /renewcaptcha                          : get token and captcha image
 #   * POST /sessions/<type>/<session-id>?auth    : authenticate with a fixed
 #                                                  sessionId
 #   * Note that the "getCookie" method (authentification via SOAP) exists for
@@ -27,12 +31,14 @@
 #                                                  n° <cfgNum>
 #   * GET /config/<latest|cfgNum>/<key>           : get conf key value
 #   * GET /config/<latest|cfgNum>?full            : get the full configuration
-#   where <type> is the session type ("global" for SSO session)
+#   where <type> is the session type ("global" for SSO session or "persistent")
+#   * GET /error/<lang>/<errNum>                  : get <errNum> message reference and errors file <lang>.json
+#   Return 'en' error file if no <lang> specified
 #
 # - Authorizations for connected users (always):
 #   * GET /mysession/?whoami                               : get "my" uid
 #   * GET /mysession/?authorizationfor=<base64-encoded-url>: ask if url is
-#                                                            authorizated
+#                                                            authorized
 #   * PUT /mysession/<type>                                : update some
 #                                                            persistent data
 #                                                            (restricted)
@@ -48,9 +54,10 @@ use Mouse;
 use JSON qw(from_json to_json);
 use MIME::Base64;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.8';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin';
+extends
+  qw (Lemonldap::NG::Portal::Main::Plugin Lemonldap::NG::Portal::Lib::Captcha);
 
 has configStorage => (
     is      => 'ro',
@@ -73,7 +80,7 @@ has exportedAttr => (
                 'authenticationLevel', 'groups',
                 'ipAddr',              '_startTime',
                 '_utime',              '_lastSeen',
-                '_session_id',
+                '_session_id',         '_session_kind',
             );
             if ( my $exportedAttr = $conf->{exportedAttr} ) {
                 $exportedAttr =~ s/^\s*\+\s+//;
@@ -90,8 +97,8 @@ has exportedAttr => (
         }
     }
 );
-
-has ott => (
+has captcha => ( is => 'rw' );
+has ott     => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
@@ -108,6 +115,13 @@ sub init {
     my ($self)  = @_;
     my @parents = ('Lemonldap::NG::Portal::Main::Plugin');
     my $add     = 0;
+    if (   $self->conf->{captcha_mail_enabled}
+        || $self->conf->{captcha_login_enabled}
+        || $self->conf->{captcha_register_enabled} )
+    {
+        $self->captcha( $self->p->loadModule('::Lib::Captcha') ) or return 0;
+        $self->addUnauthRoute( renewcaptcha => 'sendCaptcha', ['GET'] );
+    }
     if ( $self->conf->{restConfigServer} ) {
         push @parents, 'Lemonldap::NG::Common::Conf::RESTServer';
         $add++;
@@ -122,11 +136,14 @@ sub init {
                 ]
             },
             ['GET'],
-        );
-        $self->addUnauthRoute(
+          )
+
+          ->addUnauthRoute(
             config => { ':cfgNum' => { '*' => 'getKey' } },
             ['GET']
-        );
+          )
+
+          ->addUnauthRoute( error => { '*' => 'getError' }, ['GET'] );
     }
     if ( $self->conf->{restSessionServer} ) {
         push @parents, 'Lemonldap::NG::Common::Session::REST';
@@ -134,46 +151,61 @@ sub init {
 
         # Methods inherited from Lemonldap::NG::Common::Session::REST
         $self->addUnauthRoute(
-            sessions => { ':sessionType' => 'session' },
+            sessions => {
+                ':sessionType' => (
+                    $self->conf->{restExportSecretKeys}
+                    ? 'rawSession'
+                    : 'session'
+                )
+            },
             ['GET']
-        );
-        $self->addUnauthRoute(
+          )
+
+          ->addUnauthRoute(
             sessions => { ':sessionType' => 'newSession' },
             ['POST']
-        );
+          )
 
-        # Methods written below
-        $self->addUnauthRoute(
+          # Methods written below
+          ->addUnauthRoute(
             sessions => { ':sessionType' => 'updateSession' },
             ['PUT']
-        );
-        $self->addUnauthRoute(
+          )
+
+          ->addUnauthRoute(
             sessions => { ':sessionType' => 'delSession' },
             ['DELETE']
-        );
+          )
 
-        $self->addAuthRoute(
+          ->addAuthRoute(
             session => { my => { ':sessionType' => 'getMyKey' } },
             [ 'GET', 'POST' ]
-        );
+          )
+
+          ->addAuthRoute(
+            sessions => { my => { ':sessionType' => 'removeSessions' } },
+            ['DELETE']
+          );
     }
 
     # Methods always available
     $self->addAuthRoute(
         mysession => { '*' => 'mysession' },
         [ 'GET', 'POST' ]
-    );
-    $self->addAuthRoute(
+      )
+
+      ->addAuthRoute(
         mysession => {
             ':sessionType' =>
               { ':key' => 'delKeyInMySession', '*' => 'delMySession' }
         },
         ['DELETE']
-    );
-    $self->addAuthRoute(
+      )
+
+      ->addAuthRoute(
         mysession => { ':sessionType' => 'updateMySession' },
         ['PUT']
-    );
+      );
     extends @parents if ($add);
     $self->setTypes( $self->conf ) if ( $self->conf->{restSessionServer} );
     return 1;
@@ -194,18 +226,45 @@ sub newSession {
     my $force = 0;
     if ( my $s = delete $infos->{__secret} ) {
         my $t;
-        if ( $t =
-                $self->conf->{cipher}->decrypt($s)
-            and $t <= time
-            and $t > time - 30 )
-        {
-            $force = 1;
+        if ( $t = $self->conf->{cipher}->decrypt($s) ) {
+            if (    $t <= time + $self->conf->{restClockTolerance}
+                and $t > time - $self->conf->{restClockTolerance} )
+            {
+                $force = 1;
+            }
+            else {
+                $self->userLogger->error( 'Clock drift between servers is'
+                      . ' beyond tolerance, force denied.' );
+            }
         }
         else {
             $self->userLogger->error('Bad key, force denied');
         }
     }
 
+    if ( $req->param('all') and not $id ) {
+        return $self->p->sendError( $req,
+            'Bad key, refuse to send all sessions', 400 )
+          unless ($force);
+        my $data = $infos->{data};
+        my $opts = $self->conf->{globalStorageOptions} || {};
+        $opts->{backend} = $self->conf->{globalStorage};
+        my $sessions;
+        if ( my $query = $req->param('search') ) {
+            my ( $field, @values ) = split /,/, $query;
+            $sessions = Lemonldap::NG::Common::Apache::Session->searchOn(
+                $opts, $field,
+                join( ',', @values ),
+                ( $data ? @$data : () )
+            );
+        }
+        else {
+            $sessions =
+              Lemonldap::NG::Common::Apache::Session
+              ->get_key_from_all_sessions( $opts, $data );
+        }
+        return $self->p->sendJSONresponse( $req, $sessions );
+    }
     my $session = $self->getApacheSession( $mod, $id, $infos, $force );
     return $self->p->sendError( $req, 'Unable to create session', 500 )
       unless ($session);
@@ -233,7 +292,7 @@ sub newAuthSession {
     $req->data->{password} = $req->param('password');
     $req->steps( [
             @{ $self->p->beforeAuth },
-            qw(getUser authenticate setAuthSessionInfo),
+            qw(getUser extractFormInfo authenticate setAuthSessionInfo),
             @{ $self->p->betweenAuthAndData },
             $self->p->sessionData,
             @{ $self->p->afterData },
@@ -266,12 +325,16 @@ sub updateSession {
     my $force = 0;
     if ( my $s = delete $infos->{__secret} ) {
         my $t;
-        if ( $t =
-                $self->conf->{cipher}->decrypt($s)
-            and $t <= time
-            and $t > time - 30 )
-        {
-            $force = 1;
+        if ( $t = $self->conf->{cipher}->decrypt($s) ) {
+            if (    $t <= time + $self->conf->{restClockTolerance}
+                and $t > time - $self->conf->{restClockTolerance} )
+            {
+                $force = 1;
+            }
+            else {
+                $self->userLogger->error( 'Clock drift between servers is'
+                      . ' beyond tolerance, force denied.' );
+            }
         }
         else {
             $self->userLogger->error('Bad key, force denied');
@@ -492,4 +555,43 @@ sub delKeyInMySession {
         { result => 1, count => $res, modifiedKeys => $dkey } );
 }
 
+sub getError {
+    my ( $self, $req, $lang, $errNum ) = @_;
+    $lang ||= 'en';
+    $errNum
+      ? $self->logger->debug("Send $lang error file path with error: $errNum")
+      : $self->logger->debug("Send $lang error file path");
+
+    return $self->p->sendJSONresponse(
+        $req,
+        {
+            result   => 1,
+            lang     => $lang,
+            errorNum => $errNum ? $errNum : 'all',
+            errorsFileURL =>
+              "$self->{conf}->{staticPrefix}/languages/$lang.json",
+            ( $errNum ? ( errorMsgRef => "PE$errNum" ) : () ),
+        }
+    );
+}
+
+sub removeSessions {
+    my ( $self, $req ) = @_;
+    my $glPlugin =
+      $self->p->loadedModules->{'Lemonldap::NG::Portal::Plugins::GlobalLogout'};
+    my $sessions = $glPlugin->activeSessions($req);
+    my $nbr      = $glPlugin->removeOtherActiveSessions( $req, $sessions );
+
+    return $self->p->sendJSONresponse( $req, { result => $nbr } );
+}
+
+sub sendCaptcha {
+    my ( $self, $req ) = @_;
+    $self->logger->info("User request for captcha renew");
+    my ( $token, $image ) = $self->captcha->getCaptcha($req);
+
+    return $self->p->sendJSONresponse( $req,
+        { newtoken => $token, newimage => $image } );
+}
+
 1;
